@@ -8,7 +8,7 @@
 # License   : Revised BSD License
 # -----------------------------------------------------------------------------
 # Creation  : 26-Apr-2010
-# Last mod  : 13-Feb-2012
+# Last mod  : 14-Mar-2012
 # -----------------------------------------------------------------------------
 
 """
@@ -41,7 +41,7 @@ See also:
 import base64, bz2, crypt, hashlib, os, random, sys, re, string, tempfile, subprocess, types, functools
 import fabric, fabric.api, fabric.operations, fabric.context_managers
 
-VERSION     = "0.2.3"
+VERSION     = "0.2.4"
 
 RE_SPACES   = re.compile("[\s\t]+")
 MAC_EOL     = "\n"
@@ -50,6 +50,7 @@ WINDOWS_EOL = "\r\n"
 # FIXME: MODE should be in the fabric env, as this is definitely not thread-safe
 MODE_LOCAL  = False
 MODE_SUDO   = False
+SUDO_PASSWORD = None
 DEFAULT_OPTIONS = dict(
 	package="apt"
 )
@@ -60,20 +61,27 @@ DEFAULT_OPTIONS = dict(
 # with mode_sudo():
 #     pass
 
+def sudo_password(password):
+    global SUDO_PASSWORD
+    SUDO_PASSWORD= password
+
 def mode_local():
 	"""Sets Cuisine into local mode, where run/sudo won't go through
 	Fabric's API, but directly through a popen. This allows you to
 	easily test your Cuisine scripts without using Fabric."""
-	global MODE_LOCAL
+	global MODE_LOCAL, SUDO_PASSWORD	
+	sudo_cmd = "sudo "
+	if not SUDO_PASSWORD is None:
+		sudo_cmd= "echo %s|sudo -S -p '' "%SUDO_PASSWORD
 	if MODE_LOCAL is False:
 		def custom_run( cmd ):
 			global MODE_SUDO
 			if MODE_SUDO:
-				return os.popen("sudo " + cmd).read()[:-1]
+				return os.popen(sudo_cmd + cmd).read()[:-1]
 			else:
 				return os.popen(cmd).read()[:-1]
 		def custom_sudo( cmd ):
-			return os.popen("sudo " + cmd).read()[:-1]
+			return os.popen(sudo_cmd + cmd).read()[:-1]
 		module   = sys.modules[__name__]
 		old_run  = getattr(module, "run")
 		old_sudo = getattr(module, "sudo")
@@ -97,7 +105,6 @@ def mode_remote():
 	else:
 		return False
 
-
 class mode_user(object):
 	"""Cuisine functions will be executed as the current user."""
 
@@ -141,7 +148,6 @@ def select_package( option=None ):
 		fabric.api.env["option_package"] = option
 	return (fabric.api.env["option_package"], supported)
 
-
 # =============================================================================
 #
 # RUN/SUDO METHODS
@@ -230,7 +236,6 @@ def dispatch(prefix=None):
 	else:
 		return dispatch_wrapper
 
-
 # =============================================================================
 #
 # TEXT PROCESSING
@@ -378,11 +383,15 @@ def file_write(location, content, mode=None, owner=None, group=None, sudo=None):
 	os.write(fd, content)
 	# Upload the content if necessary
 	if not file_exists(location) or sig != file_sha256(location):
-		if MODE_SUDO: sudo = MODE_SUDO
-		try:
-			fabric.operations.put(local_path, location, use_sudo=sudo)
-		except Exception, e:
-			print "cuisine.file_write exception:"
+		if MODE_LOCAL:
+			run('cp "%s" "%s"'%(local_path,location))
+		else:
+			if MODE_SUDO:
+				sudo = MODE_SUDO
+			try:
+				fabric.operations.put(local_path, location, use_sudo=sudo)
+			except Exception, e:
+				print "cuisine.file_write exception:"
 	# Remove the local temp file
 	os.close(fd)
 	os.unlink(local_path)
@@ -408,7 +417,13 @@ def file_upload(remote, local, sudo=None):
 	sig     = hashlib.sha256(content).hexdigest()
 	if MODE_SUDO: sudo = MODE_SUDO
 	if not file_exists(remote) or sig != file_sha256(remote):
-		fabric.operations.put(local, remote, use_sudo=sudo)
+		if MODE_LOCAL:
+			if sudo:
+				sudo('cp "%s" "%s"'%(local,remote))
+			else:
+				run('cp "%s" "%s"'%(local,remote))
+		else:
+			fabric.operations.put(local, remote, use_sudo=sudo)
 
 def file_update(location, updater=lambda x: x):
 	"""Updates the content of the given by passing the existing
@@ -423,14 +438,12 @@ def file_update(location, updater=lambda x: x):
 	assert file_exists(location), "File does not exists: " + location
 	new_content = updater(file_read(location))
 	# assert type(new_content) in (str, unicode, fabric.operations._AttributeString), "Updater must be like (string)->string, got: %s() = %s" %  (updater, type(new_content))
-	run('echo "%s" | base64 -d > "%s"' %
-		(base64.b64encode(new_content), location))
+	run('echo "%s" | base64 -d > "%s"' % (base64.b64encode(new_content), location))
 
 def file_append(location, content, mode=None, owner=None, group=None):
 	"""Appends the given content to the remote file at the given
 	location, optionally updating its mode/owner/group."""
-	run('echo "%s" | base64 -d >> "%s"' %
-		(base64.b64encode(content), location))
+	run('echo "%s" | base64 -d >> "%s"' % (base64.b64encode(content), location))
 	file_attribs(location, mode, owner, group)
 
 def file_unlink(path):
@@ -452,16 +465,14 @@ def file_link(source, destination, symbolic=True, mode=None, owner=None, group=N
 
 def file_sha256(location):
 	"""Returns the SHA-256 sum (as a hex string) for the remote file at the given location."""
-	return run('sha256sum "%s" | cut -d" " -f1' % (location))
-
-# TODO: From McCoy's version, consider merging
-# def file_append( location, content, use_sudo=False, partial=False, escape=True):
-#       """Wrapper for fabric.contrib.files.append."""
-#       fabric.contrib.files.append(location, content, use_sudo, partial, escape)
+	# NOTE: In some cases, sudo can output errors in here -- but the errors will
+	# appear before the result, so we simply split and get the last line to
+	# be on the safe side.
+	return run('sha256sum "%s" | cut -d" " -f1' % (location)).split("\n")[-1]
 
 # =============================================================================
 #
-# DIRECTOR OPERATIONS
+# DIRECTORY OPERATIONS
 #
 # =============================================================================
 
@@ -540,7 +551,6 @@ def package_install_apt(package, update=False):
 		package = " ".join(package)
 	sudo("apt-get --yes install %s" % (package))
 
-
 def package_ensure_apt(package, update=False):
 	status = run("dpkg-query -W -f='${Status}' %s ; true" % package)
 	if status.find("not-installed") != -1 or status.find("installed") == -1:
@@ -560,7 +570,6 @@ def command_check(command):
 	"""Tests if the given command is available on the system."""
 	return run("which '%s' >& /dev/null && echo OK ; true" % command).endswith("OK")
 
-
 def command_ensure(command, package=None):
 	"""Ensures that the given command is present, if not installs the
 	package with the given name, which is the same as the command by
@@ -647,6 +656,14 @@ def user_ensure(name, passwd=None, home=None, uid=None, gid=None, shell=None):
 		if passwd:
 			user_passwd(name, passwd)
 
+def user_remove(name, rmhome=None):
+	"""Removes the user with the given name, optionally 
+	removing the home directory and mail spool."""
+	options = ["-f"]
+	if rmhome:
+		options.append("-r")
+	sudo("userdel %s '%s'" % (" ".join(options), name))
+
 # =============================================================================
 #
 # GROUP OPERATIONS
@@ -789,7 +806,4 @@ def locale_ensure(locale):
 for option, value in DEFAULT_OPTIONS.items():
 	eval("select_" + option)(value)
 
-
-
-
 # EOF - vim: ts=4 sw=4 noet
